import json

from bs4 import BeautifulSoup
from colorama import Fore
from yarl import URL

from ..base_functions import log, ssl_context, user_agent, logger
from ..data_classes import DomainItem


class CyberfileCrawler:
    def __init__(self):
        self.load_files = URL('https://cyberfile.is/account/ajax/load_files')
        self.file_details = URL('https://cyberfile.is/account/ajax/file_details')

    async def folder_nodeId(self, session, url: URL):
        try:
            async with session.get(url, ssl=ssl_context) as response:
                text = await response.text()
                soup = BeautifulSoup(text, 'html.parser')
                script_func = soup.select_one('div[class="page-container horizontal-menu with-sidebar fit-logo-with-sidebar logged-out clear-adblock"] script').text
                script_func = script_func.split('loadImages(')[-1]
                script_func = script_func.split(';')[0]
                nodeId = int(script_func.split(',')[1].replace("'", ""))
                return nodeId

        except Exception as e:
            await log("Unable to get NodeId for " + str(url))
            logger.debug(e)
            return 0

    async def get_single_contentId(self, session, url: URL):
        try:
            async with session.get(url, ssl=ssl_context) as response:
                text = await response.text()
                soup = BeautifulSoup(text, 'html.parser')
                script_funcs = soup.select('script')
                for script in script_funcs:
                    script_text = script.text
                    if script_text:
                        if "showFileInformation" in script_text:
                            parta = script_text.split("showFileInformation(")
                            parta = [x for x in parta if x[0].isdigit()][0]
                            partb = parta.split(");")[0]
                            contentId = int(partb)
                            return contentId
                return None

        except Exception as e:
            await log("Unable to get ContentId for " + str(url))
            logger.debug(e)
            return 0

    async def shared_nodeId_root_content(self, session, url: URL, page):
        try:
            # Initial page load to tell server, this is what we want.
            async with session.get(str(url), headers={'referer': str(url), "user-agent": user_agent}, ssl=ssl_context) as response:
                pass

            # get the content listings
            data = {"pageType": "nonaccountshared", "nodeId": '', "pageStart": page, "perPage": 0, "filterOrderBy": ""}
            nodes = []
            contents = []
            async with session.post(self.load_files, data=data, headers={"user-agent": user_agent}, ssl=ssl_context) as response:
                content = json.loads(await response.content.read())
                text = content['html']
                title = content['page_title']
                soup = BeautifulSoup(text.replace("\\", ""), 'html.parser')
                total_pages = int(soup.select_one('input[id=rspTotalPages]').get('value'))
                listings = soup.select("div[class=fileListing] div")
                for listing in listings:
                    title = content['page_title']
                    try:
                        title = title + '/' + listing.select_one('span[class=filename]').text
                        nodes.append((title, int(listing.get('folderid'))))
                    except (TypeError, AttributeError):
                        pass
                    try:
                        contents.append((title, int(listing.get('fileid'))))
                    except TypeError:
                        pass
            if page < total_pages:
                nodes_temp, content_temp = await self.shared_nodeId_root_content(session, url, page+1)
                nodes.extend(nodes_temp)
                contents.extend(content_temp)
            return nodes, contents

        except Exception as e:
            await log("Unable to get NodeId for " + str(url))
            logger.debug(e)
            return []

    async def shared_content(self, session, url: URL, nodeId, page, title=None):
        try:
            nodes = []
            contents = []
            data = {"pageType": "nonaccountshared", "nodeId": nodeId, "pageStart": page, "perPage": 0, "filterOrderBy": ""}
            async with session.post(self.load_files, data=data, headers={"user-agent": user_agent}, ssl=ssl_context) as response:
                content = json.loads(await response.content.read())
                text = content['html']
                title = title if title else content['page_title']
                soup = BeautifulSoup(text.replace("\\", ""), 'html.parser')
                total_pages = int(soup.select_one('input[id=rspTotalPages]').get('value'))
                listings = soup.select("div[class=fileListing] div")
                for listing in listings:
                    try:
                        nodes.append(int(listing.get('folderid')))
                    except TypeError:
                        pass
                    try:
                        contents.append((title, int(listing.get('fileid'))))
                    except TypeError:
                        pass
            if page < total_pages:
                contents.extend(await self.shared_content(session, url, nodeId, page+1, title))
            for node in nodes:
                contents.extend(await self.shared_content(session, url, node, 1, title))
            return contents

        except Exception as e:
            await log("Unable to get content for " + str(url))
            logger.debug(e)
            return []

    async def folder_content(self, session, url: URL, nodeId, page):
        data = {"pageType": "folder", "nodeId": nodeId, "pageStart": page, "perPage": 0, "filterOrderBy": ""}
        nodes = []
        contents = []
        try:
            async with session.post(self.load_files, data=data, headers={"user-agent": user_agent}, ssl=ssl_context) as response:
                content = json.loads(await response.content.read())
                text = content['html']
                title = content['page_title']
                soup = BeautifulSoup(text.replace("\\", ""), 'html.parser')
                total_pages = int(soup.select_one('input[id=rspTotalPages]').get('value'))
                listings = soup.select("div[class=fileListing] div")
                for listing in listings:
                    try:
                        nodes.append(int(listing.get('folderid')))
                    except TypeError:
                        pass
                    try:
                        contents.append((title, int(listing.get('fileid'))))
                    except TypeError:
                        pass
            if page < total_pages:
                contents.extend(await self.folder_content(session, url, nodeId, page+1))
            for node in nodes:
                contents.extend(await self.folder_content(session, url, node, 1))
            return contents

        except Exception as e:
            await log("Unable to get NodeId for " + str(url))
            logger.debug(e)
            return []

    async def get_download_links(self, session, url, contentIds):
        download_links = []
        try:
            for title, contentId in contentIds:
                data = {"u": contentId}
                async with session.post(self.file_details, data=data, headers={"user-agent": user_agent}, ssl=ssl_context) as response:
                    text = await response.text()
                    soup = BeautifulSoup(text.replace("\\", ""), 'html.parser')
                    menu = soup.select_one('ul[class="dropdown-menu dropdown-info account-dropdown-resize-menu"] li a')
                    if menu:
                        html_download_text = menu.get("onclick")
                        link = URL(html_download_text.replace("openUrl('", "").replace("'); return false;", ""))
                        download_links.append((title, link))
                    button = soup.select_one('div[class="btn-group responsiveMobileMargin"] button')
                    if button:
                        html_download_text = button.get("onclick")
                        link = URL(html_download_text.replace("openUrl('", "").replace("'); return false;", ""))
                        download_links.append((title, link))
            return download_links

        except Exception as e:
            await log("Unable to get content for " + str(url))
            logger.debug(e)
            return []

    async def fetch(self, session, url: URL):
        await log("Starting scrape of " + str(url), Fore.WHITE)
        domain_obj = DomainItem("cyberfile.is", {})
        download_links = []
        if 'folder' in url.parts:
            nodeId = await self.folder_nodeId(session, url)
            contentIds = await self.folder_content(session, url, nodeId, 1)
            download_links = await self.get_download_links(session, url, contentIds)
        elif 'shared' in url.parts:
            nodeIds, contentIds = await self.shared_nodeId_root_content(session, url, 1)
            for title, nodeId in nodeIds:
                contentIds.extend(await self.shared_content(session, url, nodeId, 1, title))
            download_links = await self.get_download_links(session, url, contentIds)
        else:
            contentId = await self.get_single_contentId(session, url)
            if contentId:
                download_links = await self.get_download_links(session, url, [("Loose Cyberfile Files", contentId)])
        for title, download_link in download_links:
            await domain_obj.add_to_album(title, download_link, url)
        await log("Finished scrape of " + str(url), Fore.WHITE)
        return domain_obj



import asyncio

import aiofiles
from yarl import URL

from .crawlers.Anonfiles_Spider import AnonfilesCrawler
from .crawlers.Chibisafe_Spider import ChibisafeCrawler
from .crawlers.Coomer_Spider import CoomerCrawler
from .crawlers.Cyberfile_Spider import CyberfileCrawler
from .crawlers.Erome_Spider import EromeCrawler
from .crawlers.Gfycat_Spider import GfycatCrawler
from .crawlers.GoFile_Spider import GofileCrawler
from .crawlers.Pixeldrain_Crawler import PixelDrainCrawler
from .crawlers.Redgifs_Spider import RedGifsCrawler
from .crawlers.ShareX_Spider import ShareXCrawler
from .crawlers.Thotsbay_Spider import ThotsbayCrawler
from .base_functions import log
from .data_classes import CascadeItem


class ScrapeMapper():
    def __init__(self, *, session, include_id=False, thotsbay_auth=None, separate_posts=False):
        self.include_id = include_id
        self.separate_posts = separate_posts
        self.thotsbay_auth = thotsbay_auth
        self.session = session
        self.Cascade = CascadeItem({})

        self.anonfiles_crawler = None
        self.chibisafe_crawler = None
        self.coomer_crawler = None
        self.cyberfile_crawler = None
        self.erome_crawler = None
        self.gfycat_crawler = None
        self.gofile_crawler = None
        self.pixeldrain_crawler = None
        self.redgifs_crawler = None
        self.sharex_crawler = None
        self.thotsbay_crawler = None

        self.semaphore = asyncio.Semaphore(1)
        self.mapping = {"anonfiles.com": self.Anonfiles, "bunkr.is": self.Chibisafe,
                        "bunkr.to": self.Chibisafe, "coomer.party": self.coomer,
                        "cyberdrop.cc": self.Chibisafe, "cyberdrop.me": self.Chibisafe,
                        "cyberdrop.nl": self.Chibisafe, "cyberdrop.to": self.Chibisafe,
                        "cyberfile.is": self.cyberfile, "erome.com": self.Erome,
                        "gfycat.com": self.gfycat, "gofile.io": self.GoFile,
                        "jpg.church": self.ShareX, "pixeldrain.com": self.Pixeldrain,
                        "pixl.is": self.ShareX, "putme.ga": self.ShareX,
                        "putmega.com": self.ShareX, "redgifs.com": self.redgifs,
                        "socialmediagirls.com": self.ThotsBay, "thotsbay.com": self.ThotsBay}

    async def Anonfiles(self, url: URL, title=None):
        if not self.anonfiles_crawler:
            self.anonfiles_crawler = AnonfilesCrawler(
                include_id=self.include_id)
        domain_obj = await self.anonfiles_crawler.fetch(self.session, url)
        if title:
            await domain_obj.append_title(title)
        await self.Cascade.add_albums(domain_obj)

    async def Chibisafe(self, url: URL, title=None):
        if not self.chibisafe_crawler:
            self.chibisafe_crawler = ChibisafeCrawler(
                include_id=self.include_id)
        domain_obj = await self.chibisafe_crawler.fetch(self.session, url)
        if title:
            await domain_obj.append_title(title)
        await self.Cascade.add_albums(domain_obj)

    async def coomer(self, url: URL, title=None):
        if not self.coomer_crawler:
            self.coomer_crawler = CoomerCrawler(include_id=self.include_id)
        domain_obj = await self.coomer_crawler.fetch(self.session, url)
        if title:
            await domain_obj.append_title(title)
        await self.Cascade.add_albums(domain_obj)

    async def cyberfile(self, url: URL, title=None):
        if not self.cyberfile_crawler:
            self.cyberfile_crawler = CyberfileCrawler()
        async with self.semaphore:
            domain_obj = await self.cyberfile_crawler.fetch(self.session, url)
        if title:
            await domain_obj.append_title(title)
        await self.Cascade.add_albums(domain_obj)

    async def Erome(self, url: URL, title=None):
        if not self.erome_crawler:
            self.erome_crawler = EromeCrawler(include_id=self.include_id)
        domain_obj = await self.erome_crawler.fetch(self.session, url)
        if title:
            await domain_obj.append_title(title)
        await self.Cascade.add_albums(domain_obj)

    async def GoFile(self, url: URL, title=None):
        if not self.gofile_crawler:
            try:
                self.gofile_crawler = GofileCrawler()
            except:
                await log("Couldn't start the GoFile crawler")
                return
        domain_obj = await self.gofile_crawler.fetch(self.session, url)
        if title:
            await domain_obj.append_title(title)
        await self.Cascade.add_albums(domain_obj)

    async def gfycat(self, url: URL, title=None):
        if not self.gfycat_crawler:
            self.gfycat_crawler = GfycatCrawler(
                scraping_mapper=self, session=self.session)
        content_url = await self.gfycat_crawler.fetch(self.session, url)
        if content_url:
            if title:
                await self.Cascade.add_to_album("gfycat.com", f"{title}/gifs", content_url, url)
            else:
                await self.Cascade.add_to_album("gfycat.com", "gifs", content_url, url)

    async def Pixeldrain(self, url: URL, title=None):
        if not self.pixeldrain_crawler:
            self.pixeldrain_crawler = PixelDrainCrawler()
        domain_obj = await self.pixeldrain_crawler.fetch(self.session, url)
        if title:
            await domain_obj.append_title(title)
        await self.Cascade.add_albums(domain_obj)

    async def redgifs(self, url: URL, title=None):
        if not self.redgifs_crawler:
            self.redgifs_crawler = RedGifsCrawler(scraping_mapper=self, session=self.session)
        content_url = await self.redgifs_crawler.fetch(self.session, url)
        if content_url:
            if title:
                await self.Cascade.add_to_album("redgifs.com", f"{title}/gifs", content_url, url)
            else:
                await self.Cascade.add_to_album("redgifs.com", "gifs", content_url, url)

    async def ShareX(self, url: URL, title=None):
        if not self.sharex_crawler:
            self.sharex_crawler = ShareXCrawler(include_id=self.include_id)
        domain_obj = await self.sharex_crawler.fetch(self.session, url)
        if title:
            await domain_obj.append_title(title)
        await self.Cascade.add_albums(domain_obj)

    async def ThotsBay(self, url: URL, title=None):
        if not self.thotsbay_crawler:
            self.thotsbay_crawler = ThotsbayCrawler(
                include_id=self.include_id, auth=self.thotsbay_auth,
                scraping_mapper=self, session=self.session, separate_posts=self.separate_posts)
        await self.Cascade.extend(await self.thotsbay_crawler.fetch(self.session, url))

    async def map_url(self, url_to_map: URL, title=None):
        for key, value in self.mapping.items():
            if key in url_to_map.host:
                await value(url=url_to_map, title=title)
                return
        await log(str(url_to_map) + " is not supported currently.")
        async with aiofiles.open("./Unsupported_Urls.txt", mode='a') as f:
            await f.write(str(url_to_map)+"\n")
